' ObservableNotifiableCollection.cs by Charles Petzold, December 2008

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel

Namespace DataLibrary
	Public Class ObservableNotifiableCollection(Of T As INotifyPropertyChanged)
		Inherits ObservableCollection(Of T)
		Public Event ItemPropertyChanged As ItemPropertyChangedEventHandler
		Public Event CollectionCleared As EventHandler

		Protected Overrides Sub OnCollectionChanged(ByVal args As NotifyCollectionChangedEventArgs)
			MyBase.OnCollectionChanged(args)

			If args.NewItems IsNot Nothing Then
				For Each item As INotifyPropertyChanged In args.NewItems
					AddHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
				Next item
			End If

			If args.OldItems IsNot Nothing Then
				For Each item As INotifyPropertyChanged In args.OldItems
					RemoveHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
				Next item
			End If
		End Sub

		Private Sub OnItemPropertyChanged(ByVal sender As Object, ByVal args As PropertyChangedEventArgs)
			RaiseEvent ItemPropertyChanged(Me, New ItemPropertyChangedEventArgs(sender, args.PropertyName))
		End Sub

		Protected Overrides Sub ClearItems()
			For Each item As INotifyPropertyChanged In Items
				RemoveHandler item.PropertyChanged, AddressOf OnItemPropertyChanged
			Next item

			RaiseEvent CollectionCleared(Me, EventArgs.Empty)

			MyBase.ClearItems()
		End Sub
	End Class
End Namespace
