' ScatterPlotRender.cs by Charles Petzold, December 2008

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media

Namespace DataLibrary
	Public Class ScatterPlotRender
		Inherits FrameworkElement
        Public Shared ReadOnly ItemsSourceProperty As DependencyProperty = DependencyProperty.Register("ItemsSource", GetType(ObservableNotifiableCollection(Of DataPoint)), GetType(ScatterPlotRender), New PropertyMetadata(AddressOf OnItemsSourceChanged))

		Public Shared ReadOnly BrushesProperty As DependencyProperty = DependencyProperty.Register("Brushes", GetType(Brush()), GetType(ScatterPlotRender), New FrameworkPropertyMetadata(Nothing, FrameworkPropertyMetadataOptions.AffectsRender))

		Public Shared ReadOnly BackgroundProperty As DependencyProperty = Panel.BackgroundProperty.AddOwner(GetType(ScatterPlotRender))

		Public Property ItemsSource() As ObservableNotifiableCollection(Of DataPoint)
			Set(ByVal value As ObservableNotifiableCollection(Of DataPoint))
				SetValue(ItemsSourceProperty, value)
			End Set
			Get
				Return CType(GetValue(ItemsSourceProperty), ObservableNotifiableCollection(Of DataPoint))
			End Get
		End Property

		Public Property Brushes() As Brush()
			Set(ByVal value As Brush())
				SetValue(BrushesProperty, value)
			End Set
			Get
				Return CType(GetValue(BrushesProperty), Brush())
			End Get
		End Property

		Public Property Background() As Brush
			Set(ByVal value As Brush)
				SetValue(BackgroundProperty, value)
			End Set
			Get
				Return CType(GetValue(BackgroundProperty), Brush)
			End Get
		End Property

		Private Shared Sub OnItemsSourceChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
			TryCast(obj, ScatterPlotRender).OnItemsSourceChanged(args)
		End Sub

		Private Sub OnItemsSourceChanged(ByVal args As DependencyPropertyChangedEventArgs)
			If args.OldValue IsNot Nothing Then
				Dim coll As ObservableNotifiableCollection(Of DataPoint) = TryCast(args.OldValue, ObservableNotifiableCollection(Of DataPoint))
				RemoveHandler coll.CollectionChanged, AddressOf OnCollectionChanged
				RemoveHandler coll.ItemPropertyChanged, AddressOf OnItemPropertyChanged
			End If

			If args.NewValue IsNot Nothing Then
				Dim coll As ObservableNotifiableCollection(Of DataPoint) = TryCast(args.NewValue, ObservableNotifiableCollection(Of DataPoint))
				AddHandler coll.CollectionChanged, AddressOf OnCollectionChanged
				AddHandler coll.ItemPropertyChanged, AddressOf OnItemPropertyChanged
			End If

			InvalidateVisual()
		End Sub

		Private Sub OnCollectionChanged(ByVal sender As Object, ByVal args As NotifyCollectionChangedEventArgs)
			InvalidateVisual()
		End Sub

		Private Sub OnItemPropertyChanged(ByVal sender As Object, ByVal args As ItemPropertyChangedEventArgs)
			InvalidateVisual()
		End Sub

		Protected Overrides Sub OnRender(ByVal dc As DrawingContext)
			dc.DrawRectangle(Background, Nothing, New Rect(RenderSize))

			If ItemsSource Is Nothing OrElse Brushes Is Nothing Then
				Return
			End If

			For Each dataPoint As DataPoint In ItemsSource
				dc.DrawEllipse(Brushes(dataPoint.Type), Nothing, New Point(RenderSize.Width * dataPoint.VariableX, RenderSize.Height * dataPoint.VariableY), 1, 1)
			Next dataPoint
		End Sub
	End Class
End Namespace
